/* libsvg-cairo - Render SVG documents using the cairo library
 *
 * Copyright  2002 USC/Information Sciences Institute
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of
 * Information Sciences Institute not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.  Information Sciences Institute
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied
 * warranty.
 *
 * INFORMATION SCIENCES INSTITUTE DISCLAIMS ALL WARRANTIES WITH REGARD
 * TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL INFORMATION SCIENCES
 * INSTITUTE BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Carl Worth <cworth@isi.edu>
 */

#ifndef SVG_CAIRO_INTERNAL_H
#define SVG_CAIRO_INTERNAL_H

#include "svg-cairo.h"
#include "svg-cairo-version.h"

/* XXX: What should this actually be? */
#define SVG_CAIRO_FONT_FAMILY_DEFAULT "verdana"

typedef struct svg_cairo_pt {
    double x;
    double y;
} svg_cairo_pt_t;

typedef struct svg_cairo_state {
    cairo_surface_t *child_surface;

    svg_paint_t fill_paint;
    svg_paint_t stroke_paint;
    double fill_opacity;
    double stroke_opacity;

    char *font_family;
    double font_size;
    svg_font_style_t font_style;
    unsigned int font_weight;
    int font_dirty;

    double *dash;
    int num_dashes;
    double dash_offset;

    double opacity;

    unsigned int viewport_width;
    unsigned int viewport_height;

    svg_text_anchor_t text_anchor;

    struct svg_cairo_state *next;

    /* XXX: This is a bad hack. The pattern handling support needs to
       be sorted out properly. */
    int in_set_pattern_HACK;
} svg_cairo_state_t;

struct svg_cairo {
    svg_t *svg;
    cairo_t *cr;

    svg_cairo_state_t *state;

    unsigned int viewport_width;
    unsigned int viewport_height;
};

/* svg_cairo_sprintf_alloc.c */

int
_svg_cairo_sprintf_alloc (char **str, const char *fmt, ...);

int
_svg_cairo_vsprintf_alloc (char **str, const char *fmt, va_list ap);

/* svg_cairo_state.c */

svg_cairo_status_t
_svg_cairo_state_create (svg_cairo_state_t **state);

svg_cairo_status_t
_svg_cairo_state_init (svg_cairo_state_t *state);

svg_cairo_status_t
_svg_cairo_state_init_copy (svg_cairo_state_t *state, const svg_cairo_state_t *other);

svg_cairo_status_t
_svg_cairo_state_deinit (svg_cairo_state_t *state);

svg_cairo_status_t
_svg_cairo_state_destroy (svg_cairo_state_t *state);

svg_cairo_state_t *
_svg_cairo_state_push (svg_cairo_state_t *state);

svg_cairo_state_t *
_svg_cairo_state_pop (svg_cairo_state_t *state);


#endif
